package models

import (
	"git.nobla.cn/golang/aeus-admin/pb"
	"git.nobla.cn/golang/aeus-admin/types"
	restTypes "git.nobla.cn/golang/rest/types"
)

type (
	User struct {
		pb.UserModel
	}
	Menu struct {
		pb.MenuModel
	}
	Login struct {
		pb.LoginModel
	}
	Department struct {
		pb.DepartmentModel
	}
	Role struct {
		pb.RoleModel
	}
	Permission struct {
		pb.PermissionModel
	}
	RolePermission struct {
		pb.RolePermissionModel
	}

	Setting struct {
		pb.SettingModel
	}
)

func (m *User) GetMenu() *types.Menu {
	return &types.Menu{
		Name:   "OrganizeUser",
		Parent: "Organize",
	}
}

func (m *Login) GetMenu() *types.Menu {
	return &types.Menu{
		Name:   "OrganizeLogin",
		Parent: "Organize",
	}
}

func (m *Login) Scenario() []string {
	return []string{restTypes.ScenarioList}
}

func (m *Menu) GetMenu() *types.Menu {
	return &types.Menu{
		Name:   "OrganizeMenu",
		Parent: "Organize",
	}
}

func (m *Menu) ModelPermissions() map[string]string {
	return map[string]string{
		"organize:permission:list": "权限",
	}
}

func (m *Department) GetMenu() *types.Menu {
	return &types.Menu{
		Name:   "OrganizeDepartment",
		Parent: "Organize",
	}
}

func (m *Role) GetMenu() *types.Menu {
	return &types.Menu{
		Name:   "OrganizeRole",
		Parent: "Organize",
	}
}

func (m *Role) ModelPermissions() map[string]string {
	return map[string]string{
		"organize:role_permission:list": "权限",
	}
}

func (m *Permission) GetMenu() *types.Menu {
	return &types.Menu{
		Name:     "OrganizePermission",
		Parent:   "Organize",
		Hidden:   true,
		Uri:      "/organize/menu/permission/:id",
		ViewPath: "../views/organize/menu/Permission.vue",
	}
}

func (m *Permission) GetPermission(s string) string {
	return "organize:permission:list"
}

func (m *RolePermission) GetMenu() *types.Menu {
	return &types.Menu{
		Name:     "OrganizeRolePermission",
		Parent:   "Organize",
		Hidden:   true,
		Uri:      "/organize/role/permission/:id",
		ViewPath: "../views/organize/role/Permission.vue",
	}
}

func (m *RolePermission) GetPermission(s string) string {
	return "organize:role_permission:list"
}

func (m *Setting) GetMenu() *types.Menu {
	return &types.Menu{
		Name:   "SystemSetting",
		Parent: "System",
	}
}

func (m *Setting) ModuleName() string {
	return "system"
}
